package accrue.analysis.goals;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import polyglot.ast.NodeFactory;
import polyglot.frontend.Job;
import polyglot.frontend.Scheduler;
import polyglot.frontend.goals.Goal;
import polyglot.frontend.goals.VisitorGoal;
import polyglot.types.TypeSystem;
import polyglot.visit.ExpressionFlattener;
import accrue.AccrueScheduler;

public class ExpressionFlattenerGoal extends VisitorGoal {
    public static Goal create(Scheduler scheduler, Job job) {
        NodeFactory nf = job.extensionInfo().nodeFactory();
        TypeSystem ts = job.extensionInfo().typeSystem();
        return scheduler.internGoal(new ExpressionFlattenerGoal(job, nf, ts));
    }

    protected ExpressionFlattenerGoal(Job job, NodeFactory nf, TypeSystem ts) {
        super(job, new ExpressionFlattener(job, ts, nf));
    }

    @Override
    public Collection<Goal> prerequisiteGoals(Scheduler scheduler) {
        List<Goal> l = new ArrayList<Goal>();
        l.add(scheduler.ReachabilityChecked(job));
        l.add(scheduler.ConstantsChecked(job));
        l.add(scheduler.ExceptionsChecked(job));
        l.add(scheduler.ExitPathsChecked(job));
        l.add(scheduler.InitializationsChecked(job));
        l.add(scheduler.ConstructorCallsChecked(job));
        l.add(scheduler.ForwardReferencesChecked(job));
        l.add(((AccrueScheduler) scheduler).helper()
                                           .MakeNarrowingAssignmentsExplicit(job));
        l.add(((AccrueScheduler) scheduler).helper().InlineFinalConstants(job));
        return l;
    }
}
